#[repr(u8)]pub enum Pad {
Show 17 variants
South = 0,
East = 1,
West = 2,
North = 3,
LeftBumper = 4,
RightBumper = 5,
LeftTrigger = 6,
RightTrigger = 7,
Select = 8,
Start = 9,
Guide = 10,
LeftStick = 11,
RightStick = 12,
DPadUp = 13,
DPadDown = 14,
DPadLeft = 15,
DPadRight = 16,
}Expand description
A button of the standard gamepad layout, by its position rather than by what a maker prints on it.
Variants
South = 0
The South pad button.
East = 1
The East pad button.
West = 2
The West pad button.
North = 3
The North pad button.
LeftBumper = 4
The LeftBumper pad button.
RightBumper = 5
The RightBumper pad button.
LeftTrigger = 6
The LeftTrigger pad button.
RightTrigger = 7
The RightTrigger pad button.
Select = 8
The Select pad button.
Start = 9
The Start pad button.
Guide = 10
The Guide pad button.
LeftStick = 11
The LeftStick pad button.
RightStick = 12
The RightStick pad button.
DPadUp = 13
The DPadUp pad button.
DPadDown = 14
The DPadDown pad button.
DPadLeft = 15
The DPadLeft pad button.
DPadRight = 16
The DPadRight pad button.