pub enum Switch {
Key(Key),
Mouse(MouseButton),
}Expand description
A control the window sends events for, rather than one that is polled.
Variants
Key(Key)
A key, by physical position.
Mouse(MouseButton)
A mouse button, which the first touch of a touch screen also presses.