InputActions

Trait InputActions 

pub trait InputActions {
    type Button: InputButtonAction;
    type Axis: InputAxisAction;
    type Axis2: InputAxis2Action;
}
The three vocabularies one game plays with, named together as Game::InputActions.

Fill a kind a game has no verbs of with NoInputButtons, NoInputAxes or NoInputAxes2.

Required Associated Types

type Button: InputButtonAction

The vocabulary whose actions read back as held or not.

type Axis: InputAxisAction

The vocabulary whose actions read back a number.

type Axis2: InputAxis2Action

The vocabulary whose actions read back a vector.

Dyn Compatibility

This trait is dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety".

Implementors