pub trait SaveValue: Kept { }Expand description
Save key value types: bool, i64, f64 or String, and nothing
else.
A game’s own type is kept as one of these: an enum as a number, a whole state as text.
Dyn Compatibility
This trait is not dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety".