Module ray
mirage_
engine
0.2.0
Module ray
Module Items
Structs
In crate mirage_
engine
mirage_engine
Module
ray
The ray a pixel lies on, and the shapes a game can hit with it.
Structs

Plane
A plane in the world, defined by a point on it and its normal.
Ray
A point in the world and the direction it looks.